#!/bin/bash
# illich.cgi 
# created and copyrighted: 06 Mar 11
# author: Evsiz (evsiz@this-voice.org)
# The author releases this file under the GNU GPL
# license. See GNU's COPYING file for specifics.
#

# User Consts
CGI=/cgi-bin/illich.cgi
ROOT=illich  # cgi-bin/illich
BANNED_IPS="x"  # space between IPs - keep x at end of list
TARBALL_DIR=data/private # dir for tarball 
SELF="169.244.6.228 x"  # your IP used for admin access
if [ -f .local  ]; then SELF="127.0.0.1 x";  fi

# Security
REMOTE=$HTTP_X_CLUSTER_CLIENT_IP 
if [ "$REMOTE" == "" ]; then REMOTE=$REMOTE_ADDR; fi

BANNED=`echo $BANNED_IPS | awk 'BEGIN { RS = " " }; { \
    if ( index( "'$REMOTE'", $0 ) == 1 ) { print "banned"; exit } }'`
if [ "$BANNED" == "banned" ]; then echo "No Orcs Allowed"; exit; fi

# Methods
REQUEST=`echo  "$REQUEST_URI" | cut -d"?" -f2 | tr -d "*"`

parse_val () {
    echo $1 | awk 'BEGIN { RS = "&" } ; \
	{ if (index($0, "'$2'") == 1) { split($0, tmp, "="); print tmp[2]; exit; }};'
}

decode () {
    arg=`echo $1 | awk '{ 
       gsub(/\%0D\%0A/, "\r") ;
       gsub(/\%0d\%0a/, "\r") ;
       gsub(/\x2B/, " ") ; 
       gsub(/\x2b/, " ") ; 
       print
    }'`
    i="0"
    while [ "$i" -lt ${#arg} ]; do
        c0=${arg:$i:1}
        if [ "x$c0" = "x%" ]; then
            c1=${arg:$((i+1)):1}
            c2=${arg:$((i+2)):1}
            printf "\x$c1$c2"
            i=$((i+3))
        else
            echo -n "$c0"
            i=$((i+1))
        fi
    done
}

#
# remove files older than 30 days
# Wish I could use: find $ROOT/ -ctime 30 -delete >/dev/null
#

#preserve my own files
touch $ROOT/127.0.0.1*

NOW=`date +%s`
THIRTY=259200    #30 days in seconds
F=`ls -tr $ROOT`
for f in $F; do
    THEN=`stat -c%Y $ROOT/$f`
    TIME=`expr $NOW - $THEN`
    if [ $TIME -gt $THIRTY ]; then
        rm $ROOT/$f
    else
        break
    fi
    done

# end 30-day code


MODE=$(parse_val $REQUEST "mode")
# top
echo "Content-Type: text/html

<html><head><title>The Illich Post</title></head>
<body><h4><a href="$CGI">
The Illich Post</a></h4>"


# middle

# add
if [ "$MODE" == "add" ]; then
    MONTH=$(parse_val $REQUEST "month")
    if [ "$MONTH" != `date +%m` ]; then
		echo "Wrong month.<br>Entry Cancelled."
		exit
		fi
    TEXT=$(parse_val $REQUEST "content")
    HTML=`echo $TEXT | awk 'BEGIN { RS = "\a" } ; \
	{ if ($0 ~ /\%3C/) print "FOUND"  } '`
    LEN=`echo $TEXT | awk 'BEGIN { RS = "\a" } ; \
	{ if (length($0) < 1500) print "OK" } '`
    if [ "$HTML" == "FOUND" ]; then
		echo "No HTML tags permitted.<br>Entry Cancelled."
    elif [ "$LEN" != "OK" ]; then
		echo "Submission over 1500 bytes.<br>Entry Cancelled."
    else
        # write entry, append link in log
		FNAME=$REMOTE"_"`date +%Y%j%H%M`
		CITY=$(decode $(parse_val $REQUEST "city"))
		NAME=$(decode $(parse_val $REQUEST "name"))
		EMAIL=$(parse_val $REQUEST "email")
		EMAIL=`echo $EMAIL | sed 's_@_-at-_'`
		EMAIL=`echo $EMAIL | sed 's_%40_-at-_'`
		EMAIL=`echo $EMAIL | sed 's_\._-dot-_g'`
		TEXT=$(decode $TEXT)
		TEXT="<p>Town:  "$CITY"
		<p>"$NAME" ----- "$EMAIL"
		<p><small><pre>"$TEXT"</pre></small>
		<p>Posted by "$REMOTE

		# creation page
		echo "<title>New Entry Creation</title> 
		<body bgcolor=\"#dddddd\">
		<h4>New Entry Creation</h4><hr>"
		echo $TEXT

                echo "
<form method=\"get\" action="$CGI">
<input type=\"hidden\" name=\"fname\" value=\"$FNAME\">
<input type=\"hidden\" name=\"content\" value=\"$TEXT\">
<p>If the post here is correct: <input type=\"submit\" value=\"Post It\">.
<p>Or you can use the back button to go back and change it.
</form>
Or return to <a href="$CGI">index</a>."
    fi

# search
elif [ "$MODE" == "search" ] ; then
	STERM=$(parse_val $REQUEST "sterm")
	STERM=$(decode $STERM)
    RESULT=`grep -l "$STERM" $ROOT/* 2> /dev/null`
    if [ $? -eq 0 ]; then
		for entry in $RESULT; do
			echo "<hr>"
			cat $entry
			done
    else
		echo "<hr><h4>No results in search</h4>"
    fi  


# all
elif [ "$MODE" == "all" ] ; then
	ENTRIES=`ls -1 $ROOT/`	
	for entry in $ENTRIES; do
		echo "<hr>"
		cat $ROOT/$entry
		done

# admin
elif [ "$MODE" == "admin" ] ; then
    # Validate IP for admin
#    IS_ADMIN=`echo $SELF | awk 'BEGIN { RS = " " }; \
    IS_ADMIN=`echo "$REMOTE x" | awk 'BEGIN { RS = " " }; \
	{  if ( index( "'$REMOTE'", $0 ) == 1 ) { print "okay"; exit } }'`
    if [ "$IS_ADMIN" == "okay" ]; then
		TYPE=$(parse_val $REQUEST "type")
		if [ "$TYPE" == "delete" ]; then
			PAGE=$(parse_val $REQUEST "page")
			PAGE=`echo $PAGE | tr -d "/?*"`
			echo "deleting "$PAGE
			rm $ROOT/$PAGE || echo " - deletion failed" 
		elif [ "$TYPE" == "archive" ]; then
			tar cjf $TARBALL_DIR/illich.tar.bz2 $ROOT && \
				echo "illich archived" || echo "archive failed"
		else
			LOG=illich.log
			echo "<title>illich log</title>
			<h4>illich log</h4>
			`date +%c`<hr><pre>"
			echo "   - end of data - " >> $LOG
			cat $LOG && rm $LOG
			echo "</pre>
<p><a href=\""$CGI"?mode=admin&type=archive\">Archive</a> all entries?"
		fi
		exit
    else
		echo "No Goblins Allowed"
		exit
    fi


# default
else
        # check for new post
        FNAME=$(decode $(parse_val $REQUEST "fname"))
        if [ "$FNAME" != "" ] && [ ! -f $ROOT/$FNAME ]; then
            TEXT=$(decode $(parse_val $REQUEST "content"))

    	    # write to file
	    echo $TEXT | awk 'BEGIN { RS = "\r" } \
		{   gsub("\r", "\n") ; 
			print $0
		}' > $ROOT/$FNAME  
		# ivan log
		echo $TEXT"<p>"$FNAME"
<br><a href="$CGI"?mode=admin&type=delete&page="$FNAME">delete</a><br>" >> `pwd`/illich.log
        fi
	# header REQUIRED
	cat $ROOT/.header

echo "<hr>
<form method=\"get\" action="$CGI">
<input type=\"hidden\" name=\"mode\" value=\"add\">
<p>Town: <input type=\"text\" name=\"city\" size=\"32\" maxlength=\"32\">
<p>Name: <input type=\"text\" name=\"name\" size=\"32\" maxlength=\"64\">
<p>Email: <input type=\"text\" name=\"email\" size=\"64\" maxlength=\"64\">
<p>What you have to offer:
<p><textarea name=\"content\" rows=\"20\" cols=\"50\"></textarea>

<p>Just to make sure you are actually here, if January is 01
and February is 02, what month is it now? 
<input type=\"text\" name=\"month\" size=\"2\" maxlength=\"2\">

<p>Think Twice and Click Once: <input type=\"submit\" value=\"Submit\">
</form>"

fi # end main if

# bottom
COUNT=`ls -1 $ROOT | wc -l`


echo "<hr>
<p><form method=\"get\" action=\""$CGI"\">
<h4>Search Posts</h4>
<h5>Teachers in the database: "$COUNT"</h5>
<p><input type=\"hidden\" name=\"mode\" value=\"search\">
<p>Search Term: <input type=\"text\" name=\"sterm\" size=\"32\" maxlength=\"32\">
<p>Think Twice and Click Once: <input type=\"submit\" value=\"Submit\">
</form>
<p> Or see <a href=\""$CGI"?mode=all\">all</a> entries.
<p align=right><a href="/index.html">Home</a>
<hr></body></html>"
